package com.jiuqi.grid;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.util.Stream;
import com.jiuqi.util.StreamException;
import java.io.Serializable;

public class BitList implements Serializable {
  private byte[] bits;
  
  private int count;
  
  private boolean defaultValue;
  
  public BitList(boolean dftValue) {
    this.bits = null;
    this.count = 0;
    this.defaultValue = dftValue;
  }
  
  private void setCount(int c) {
    if (c >= 0) {
      int l = (this.bits == null) ? 0 : this.bits.length;
      if (l * 8 >= c) {
        this.count = c;
        return;
      } 
      l = (c - 1) / 8 + 1;
      if (this.bits != null) {
        byte[] nb = new byte[l];
        System.arraycopy(this.bits, 0, nb, 0, this.bits.length);
        this.bits = nb;
      } else {
        this.bits = new byte[l];
      } 
      this.count = c;
    } 
  }
  
  public void add(boolean value) {
    setCount(count() + 1);
    set(this.count - 1, value);
  }
  
  public boolean get(int index) {
    if (index >= 0 && index < this.count) {
      int b = index / 8;
      int p = index % 8;
      return ((this.bits[b] >> p & 0x1) > 0);
    } 
    return false;
  }
  
  public void set(int index, boolean value) {
    if (this.count <= index)
      setCount(index + 1); 
    int b = index / 8;
    int p = index % 8;
    if (value) {
      this.bits[b] = (byte)(this.bits[b] | 1 << p);
    } else {
      this.bits[b] = (byte)(this.bits[b] & (1 << p ^ 0xFFFFFFFF));
    } 
  }
  
  public void insert(int index, int insCount) {
    if (index >= 0 && index <= this.count) {
      setCount(this.count + insCount);
      int i;
      for (i = this.count - 1; i >= index + insCount; i--)
        set(i, get(i - insCount)); 
      for (i = index; i < index + insCount; i++)
        set(i, this.defaultValue); 
    } 
  }
  
  public void insert(int index, int insCount, boolean value) {
    if (index >= 0 && index <= this.count) {
      setCount(this.count + insCount);
      int i;
      for (i = this.count - 1; i >= index + insCount; i--)
        set(i, get(i - insCount)); 
      for (i = index; i < index + insCount; i++)
        set(i, value); 
    } 
  }
  
  public void delete(int index, int delCount) {
    if (index >= 0 && index < this.count) {
      if (index + delCount > this.count)
        delCount = this.count - index; 
      for (int i = index; i < this.count - delCount; i++)
        set(i, get(i + delCount)); 
      setCount(this.count - delCount);
    } 
  }
  
  public int count() {
    return this.count;
  }
  
  public void saveToStream(Stream stream) throws StreamException {
    if (count() > 0)
      stream.writeBuffer(this.bits, 0, (count() - 1) / 8 + 1); 
  }
  
  public void loadFromStream(Stream stream, int size) throws StreamException {
    if (size > 0) {
      this.bits = new byte[size];
      stream.readBuffer(this.bits, 0, size);
      this.count = size * 8;
    } else {
      setCount(0);
      this.bits = null;
    } 
  }
}
